/*
# Example 41-8
# A canonical Tk main program and Tk_AppInit.
 */

/* main.c */
#include <tk.h>
#include <Dbg.h>
#include <signal.h>

int Tk_AppInit(Tcl_Interp *interp);

/*
* Token and handler procedure for async event. See page 534.
*/
static Tcl_AsyncHandler sig_Token;
void Sig_Setup();
RETSIGTYPE Sig_HandleINT(int sig);
int Sig_HandleSafe(ClientData data, 
		Tcl_Interp *interp, int code);
/*
 * Needed if entering the debugger from the idle loop.
 */
static Tcl_Interp *myInterp;

/*
 * A table for command line arguments.
 */
char *myoption1 = NULL;
static int debug = 0;

static Tk_ArgvInfo argTable[] = {
	{"-myoption1", TK_ARGV_STRING, (char *) NULL,
	(char *) &myoption1, "Explain myoption1"},
	{"-debug", TK_ARGV_CONSTANT, (char *) 1, (char *) &debug,
		"Enable the Tcl debugger"},
	{"", TK_ARGV_END, },
};

main(int argc, char *argv[]) {
	Tcl_Interp *interp;
	/*
	 * Save a copy of the command line args for debugger.
	 */
	argv = Dbg_ArgcArgv(argc, argv, 1);
	/*
	 * Create an interpreter for the error message from
	 * Tk_ParseArgv. Another one is created by Tk_Main.
	 * Parse our arguments and leave the rest to Tk_Main.
	 */
	interp = Tcl_CreateInterp();
	if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv,
			argTable, 0) != TCL_OK) {
		fprintf(stderr, "%s\n", interp->result);
		exit(1);
	}
	Tcl_DeleteInterp(interp);
	/*
	 * Set up a signal handler so we can enter the Tcl 
	 * debugger.
	 */
	Sig_Setup();

	Tk_Main(argc, argv, Tk_AppInit);
	exit(0);
}

int ClockCmd(ClientData clientData,
				Tcl_Interp *interp,
				int argc, char *argv[]);

/*
 * Our pixmap image type.
 */
extern Tk_ImageType tkPixmapImageType;

int
Tk_AppInit(Tcl_Interp *interp) {
	/*
	 * Initialize packages
	 */
	if (Tcl_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Tk_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Dbg_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (debug) {
	    /*
	     * Enter the Tcl debugger before the first Tcl command.
	     */
	    Dbg_On(interp, 0);
	}
	myInterp = interp;
	/*
	 * Define application-specific commands here.
	 */
	Tcl_CreateCommand(interp, "clock", ClockCmd,
		(ClientData)Tk_MainWindow(interp),
		(Tcl_CmdDeleteProc *)NULL);

	Tk_CreateImageType(&tkPixmapImageType);
	/*
	 * Define start-up filename. This file is read in
	 * case the program is run interactively.
	 */
	Tcl_SetVar(interp, "tcl_rcFileName", "~/.mytcl",
		TCL_GLOBAL_ONLY);
	return TCL_OK;
}


/*
# Example 41-10
# A SIGINT handler.
 */

#include <signal.h>
/*
* Token and handler procedure for async event.
*/
static Tcl_AsyncHandler sig_Token;
int Sig_HandleSafe(ClientData data,
	Tcl_Interp *interp, int code);
/*
* Set up a signal handler for interrupts.
* This also registers a handler for a Tcl asynchronous
* event, which is enabled in the interrupt handler.
*/
void
Sig_Setup(interp)
	Tcl_Interp *interp;
{
	RETSIGTYPE (*oldhandler)();
	oldhandler = signal(SIGINT, Sig_HandleINT);
	if ((int)oldhandler == -1) {
		perror("signal failed");
		exit(1);
	}
	sig_Token = Tcl_AsyncCreate(Sig_HandleSafe, NULL);
#ifdef HAVE_SIGINTERRUPT
	/*
	 * Ensure that wait() kicks out on interrupt.
	 */
	siginterrupt(SIGINT, 1);
#endif
}
/*
 * Invoked upon interrupt (control-C)
 */
RETSIGTYPE
Sig_HandleINT(sig)
	int sig;
{
	Tcl_AsyncMark(sig_Token);
}
/*
* Invoked at a safe point sometime after Tcl_AsyncMark
*/
int
Sig_HandleSafe(data, interp, code)
	ClientData data;
	Tcl_Interp *interp;
	int code;
{
	if (interp == NULL) {
	    /*
	     * Called from the Tk idle loop.
	     */
	    interp = myInterp;
	}
	Dbg_On(interp, 1);						/* Enter the Tcl debugger */
}


